@@ -3,6 +3,8 @@ module Fog
     class AWS < Fog::Service
       extend Fog::AWS::CredentialFetcher::ServiceMethods
 
+      class RequestLimitExceeded < Fog::Errors::Error; end
+
       requires :aws_access_key_id, :aws_secret_access_key
       recognizes :endpoint, :region, :host, :path, :port, :scheme, :persistent, :aws_session_token, :use_iam_profile, :aws_credentials_expire_at, :instrumentor, :instrumentor_name, :version
 
@@ -120,18 +122,22 @@ module Fog
       request :describe_subnets
       request :describe_tags
       request :describe_volumes
+      request :describe_volumes_modifications
       request :describe_volume_status
       request :describe_vpcs
       request :describe_vpc_attribute
       request :describe_vpc_classic_link
+      request :describe_vpc_classic_link_dns_support
       request :detach_network_interface
       request :detach_internet_gateway
       request :detach_volume
       request :detach_classic_link_vpc
       request :disable_vpc_classic_link
+      request :disable_vpc_classic_link_dns_support
       request :disassociate_address
       request :disassociate_route_table
       request :enable_vpc_classic_link
+      request :enable_vpc_classic_link_dns_support
       request :get_console_output
       request :get_password_data
       request :import_key_pair
@@ -140,8 +146,10 @@ module Fog
       request :modify_network_interface_attribute
       request :modify_snapshot_attribute
       request :modify_subnet_attribute
+      request :modify_volume
       request :modify_volume_attribute
       request :modify_vpc_attribute
+      request :move_address_to_vpc
       request :purchase_reserved_instances_offering
       request :reboot_instances
       request :release_address
@@ -151,6 +159,7 @@ module Fog
       request :register_image
       request :request_spot_instances
       request :reset_network_interface_attribute
+      request :restore_address_to_classic
       request :revoke_security_group_egress
       request :revoke_security_group_ingress
       request :run_instances
@@ -177,15 +186,18 @@ module Fog
 
       class Mock
         MOCKED_TAG_TYPES = {
-          'ami' => 'image',
-          'i' => 'instance',
+          'acl'  => 'network_acl',
+          'ami'  => 'image',
+          'igw'  => 'internet_gateway',
+          'i'    => 'instance',
+          'rtb'  => 'route_table',
           'snap' => 'snapshot',
-          'vol' => 'volume',
-          'igw' => 'internet_gateway',
-          'acl' => 'network_acl',
-          'vpc' => 'vpc'
+          'vol'  => 'volume',
+          'vpc'  => 'vpc'
         }
 
+        VPC_BLANK_VALUE = 'none'
+
         include Fog::AWS::CredentialFetcher::ConnectionMethods
 
         def self.data
@@ -274,7 +286,7 @@ module Fog
                     "attributeName" => "supported-platforms"
                   },
                   {
-                    "values"        => ["none"],
+                    "values"        => [VPC_BLANK_VALUE],
                     "attributeName" => "default-vpc"
                   },
                   {
@@ -285,7 +297,9 @@ module Fog
                     "values"        => ["5"],
                     "attributeName" => "vpc-max-elastic-ips"
                   }
-                ]
+                ],
+                :spot_requests => {},
+                :volume_modifications => {}
               }
             end
           end
@@ -363,6 +377,79 @@ module Fog
           self.data[:account_attributes].find { |h| h["attributeName"] == "supported-platforms" }["values"] = values
         end
 
+        def default_vpc
+          vpc_id = describe_account_attributes.body["accountAttributeSet"].find{ |h| h["attributeName"] == "default-vpc" }["values"].first
+          vpc_id == VPC_BLANK_VALUE ? nil : vpc_id
+        end
+
+        def default_vpc=(value)
+          self.data[:account_attributes].find { |h| h["attributeName"] == "default-vpc" }["values"] = [value]
+        end
+
+        def setup_default_vpc!
+          return if default_vpc.present?
+
+          disable_ec2_classic
+
+          vpc_id = Fog::AWS::Mock.default_vpc_for(region)
+          self.default_vpc = vpc_id
+
+          data[:vpcs] << {
+            'vpcId' => vpc_id,
+            'state' => 'available',
+            'cidrBlock' => '172.31.0.0/16',
+            'dhcpOptionsId' => Fog::AWS::Mock.dhcp_options_id,
+            'tagSet' => {},
+            'instanceTenancy' => 'default',
+            'enableDnsSupport' => true,
+            'enableDnsHostnames' => true,
+            'isDefault' => true
+          }
+
+          internet_gateway_id = Fog::AWS::Mock.internet_gateway_id
+          data[:internet_gateways][internet_gateway_id] = {
+            'internetGatewayId' => internet_gateway_id,
+            'attachmentSet' => {
+              'vpcId' => vpc_id,
+              'state' => 'available'
+            },
+            'tagSet' => {}
+          }
+
+          data[:route_tables] << {
+            'routeTableId' => Fog::AWS::Mock.route_table_id,
+            'vpcId' => vpc_id,
+            'routes' => [
+              {
+                'destinationCidrBlock' => '172.31.0.0/16',
+                'gatewayId' => 'local',
+                'state' => 'active',
+                'origin' => 'CreateRouteTable'
+              },
+              {
+                'destinationCidrBlock' => '0.0.0.0/0',
+                'gatewayId' => internet_gateway_id,
+                'state' => 'active',
+                'origin' => 'CreateRoute'
+              }
+            ]
+          }
+
+          describe_availability_zones.body['availabilityZoneInfo'].map { |z| z['zoneName'] }.each_with_index do |zone, i|
+            data[:subnets] << {
+              'subnetId'                 => Fog::AWS::Mock.subnet_id,
+              'state'                    => 'available',
+              'vpcId'                    => vpc_id,
+              'cidrBlock'                => "172.31.#{i}.0/16",
+              'availableIpAddressCount'  => '251',
+              'availabilityZone'         => zone,
+              'tagSet'                   => {},
+              'mapPublicIpOnLaunch'      => true,
+              'defaultForAz'             => true
+            }
+          end
+        end
+
         def tagged_resources(resources)
           Array(resources).map do |resource_id|
             if match = resource_id.match(/^(\w+)-[a-z0-9]{8}/i)
@@ -386,6 +473,10 @@ module Fog
                 if self.data[:vpcs].select {|v| v['vpcId'] == resource_id }.empty?
                   raise(Fog::Service::NotFound.new("Cannot tag #{resource_id}, the vpc does not exist"))
                 end
+              when 'route_table'
+                unless self.data[:route_tables].detect { |r| r['routeTableId'] == resource_id }
+                  raise(Fog::Service::NotFound.new("Cannot tag #{resource_id}, the route table does not exist"))
+                end
               else
                 unless self.data[:"#{type}s"][resource_id]
                  raise(Fog::Service::NotFound.new("Cannot tag #{resource_id}, the #{type} does not exist"))
@@ -459,7 +550,7 @@ module Fog
           @region                 = options[:region] ||= 'us-east-1'
           @instrumentor           = options[:instrumentor]
           @instrumentor_name      = options[:instrumentor_name] || 'fog.aws.compute'
-          @version                = options[:version]     ||  '2014-10-01'
+          @version                = options[:version]     ||  '2016-11-15'
 
           @use_iam_profile = options[:use_iam_profile]
           setup_credentials(options)
@@ -523,7 +614,11 @@ module Fog
           end
         end
 
-        def _request(body, headers, idempotent, parser)
+        def _request(body, headers, idempotent, parser, retries = 0)
+
+          max_retries = 10
+
+          begin
           @connection.request({
               :body       => body,
               :expects    => 200,
@@ -532,15 +627,33 @@ module Fog
               :method     => 'POST',
               :parser     => parser
             })
-        rescue Excon::Errors::HTTPStatusError => error
-          match = Fog::AWS::Errors.match_error(error)
-          raise if match.empty?
-          raise case match[:code]
+          rescue Excon::Errors::HTTPStatusError => error
+            match = Fog::AWS::Errors.match_error(error)
+            raise if match.empty?
+            raise case match[:code]
                 when 'NotFound', 'Unknown'
                   Fog::Compute::AWS::NotFound.slurp(error, match[:message])
+                when 'RequestLimitExceeded'
+                  if retries < max_retries
+                    jitter = rand(100)
+                    waiting = true
+                    start_time = Time.now
+                    wait_time = ((2.0 ** (1.0 + retries) * 100) + jitter) / 1000.0
+                    Fog::Logger.warning "Waiting #{wait_time} seconds to retry."
+                    while waiting
+                      if Time.now - start_time >= wait_time
+                        waiting = false
+                      end
+                    end
+                    retries += 1
+                    retry
+                  else
+                    Fog::Compute::AWS::RequestLimitExceeded.slurp(error, "Max retries exceeded (#{max_retries}) #{match[:code]} => #{match[:message]}")
+                  end
                 else
                   Fog::Compute::AWS::Error.slurp(error, "#{match[:code]} => #{match[:message]}")
                 end
+          end
         end
       end
     end
