@@ -46,7 +46,7 @@ module Fog
           end
           data << "</Delete>"
 
-          headers['Content-Length'] = data.length
+          headers['Content-Length'] = data.bytesize
           headers['Content-MD5'] = Base64.encode64(OpenSSL::Digest::MD5.digest(data)).
                                    gsub("\n", '')
 
